/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"fmt"

	"github.com/Nerzal/gocloak/v7"
	"github.com/spf13/cobra"
)

func NewCreateClientCommand(rCli *RealmClient) *cobra.Command {
	// cmd represents the create command
	ro := OIDClient{}

	cmd := &cobra.Command{
		Use:   "client",
		Short: "A brief description of your command",
		Long:  "",
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.Background()
			token, err := rCli.Credentials.GetToken(ctx, rCli.URL, rCli.AuthRealm)
			if err != nil {
				return err
			}

			if err != nil {
				return err
			}

			oidc := ro.GetOIDClient()

			id, err := rCli.GetClient().CreateClient(ctx, token.AccessToken, rCli.Name, oidc)
			if err != nil {
				return err
			}

			cred, err := rCli.GetClient().GetClientSecret(ctx, token.AccessToken, rCli.Name, id)
			if err != nil {
				return err
			}

			// TODO create k8s secret with keycloak client secret
			fmt.Println(*cred.Value)
			fmt.Println(cred.SecretData)

			return nil
		},
	}

	cmd.Flags().BoolVarP(&ro.Enabled, "enabled", "", true, "Enable Realm")

	cmd.Flags().StringVarP(&ro.Name, "name", "", "", "")
	cmd.Flags().StringVarP(&ro.ClientID, "client-id", "", "", "")
	cmd.Flags().StringVarP(&ro.BaseURL, "base-url", "", "", "")
	cmd.Flags().StringVarP(&ro.RootURL, "root-url", "", "", "")
	cmd.Flags().StringVarP(&ro.ClientAuthenticatorType, "auth-type", "", "client-secret", "")

	cmd.Flags().StringSliceVarP(&ro.RedirectURIs, "redirect-uris", "", []string{}, "")
	cmd.Flags().StringSliceVarP(&ro.WebOrigins, "web-origins", "", []string{}, "")

	return cmd
}

type OIDClient struct {
	// Access:
	//   configure: "",
	//   manage: "",
	//   view: "",
	BaseURL  string
	ClientID string
	// ConsentRequired: "",
	// Description:     "",
	Enabled bool
	// Origin:          "",
	Name string
	// Secret:          "",
	RootURL                 string
	ClientAuthenticatorType string
	RedirectURIs            []string
	WebOrigins              []string
}

func (oidc *OIDClient) GetOIDClient() gocloak.Client {
	return gocloak.Client{
		BaseURL:                 &oidc.BaseURL,
		ClientID:                &oidc.ClientID,
		Enabled:                 &oidc.Enabled,
		Name:                    &oidc.Name,
		RootURL:                 &oidc.RootURL,
		ClientAuthenticatorType: &oidc.ClientAuthenticatorType,
		RedirectURIs:            &oidc.RedirectURIs,
		WebOrigins:              &oidc.WebOrigins,
	}
}
