/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"encoding/csv"
	"fmt"
	"io"
	"log"
	"os"

	"github.com/Nerzal/gocloak/v7"
	"github.com/sethvargo/go-password/password"
	"github.com/spf13/cobra"
)

type User struct {
	ID                         *string              `json:"id,omitempty"`
	CreatedTimestamp           *int64               `json:"createdTimestamp,omitempty"`
	Username                   *string              `json:"username,omitempty"`
	Enabled                    *bool                `json:"enabled,omitempty"`
	Totp                       *bool                `json:"totp,omitempty"`
	EmailVerified              *bool                `json:"emailVerified,omitempty"`
	FirstName                  *string              `json:"firstName,omitempty"`
	LastName                   *string              `json:"lastName,omitempty"`
	Email                      *string              `json:"email,omitempty"`
	FederationLink             *string              `json:"federationLink,omitempty"`
	Attributes                 *map[string][]string `json:"attributes,omitempty"`
	DisableableCredentialTypes *[]interface{}       `json:"disableableCredentialTypes,omitempty"`
	RequiredActions            *[]string            `json:"requiredActions,omitempty"`
	Access                     *map[string]bool     `json:"access,omitempty"`
	ClientRoles                *map[string][]string `json:"clientRoles,omitempty"`
	RealmRoles                 *[]string            `json:"realmRoles,omitempty"`
	Groups                     *[]string            `json:"groups,omitempty"`
	ServiceAccountClientID     *string              `json:"serviceAccountClientId,omitempty"`
	// 	Credentials                *[]CredentialRepresentation `json:"credentials,omitempty"`
}

type CredentialRepresentation struct {
	// Common part
	CreatedDate *int64  `json:"createdDate,omitempty"`
	Temporary   *bool   `json:"temporary,omitempty"`
	Type        *string `json:"type,omitempty"`
	Value       *string `json:"value,omitempty"`

	// <= v7
	Algorithm *string `json:"algorithm,omitempty"`
	//	Config            *MultiValuedHashMap `json:"config,omitempty"`
	Counter           *int32  `json:"counter,omitempty"`
	Device            *string `json:"device,omitempty"`
	Digits            *int32  `json:"digits,omitempty"`
	HashIterations    *int32  `json:"hashIterations,omitempty"`
	HashedSaltedValue *string `json:"hashedSaltedValue,omitempty"`
	Period            *int32  `json:"period,omitempty"`
	Salt              *string `json:"salt,omitempty"`

	// >= v8
	CredentialData *string `json:"credentialData,omitempty"`
	ID             *string `json:"id,omitempty"`
	Priority       *int32  `json:"priority,omitempty"`
	SecretData     *string `json:"secretData,omitempty"`
	UserLabel      *string `json:"userLabel,omitempty"`
}

func NewCreatUerCommand(rCli *RealmClient) *cobra.Command {
	// createCmd represents the create command
	user := gocloak.User{
		Enabled:  new(bool),
		Username: new(string),
		//	Password: &string{},
		Email:     new(string),
		FirstName: new(string),
		LastName:  new(string),
	}

	//	user.EmailVerified = gocloak.BoolP(true)
	//	requiredActions := []string{
	//	}

	cmd := &cobra.Command{
		Use:   "user",
		Short: "A brief description of your command",
		Long:  "",
		Run: func(cmd *cobra.Command, args []string) {
			ctx := context.Background()
			token, err := rCli.Credentials.GetToken(ctx, rCli.URL, rCli.Name)
			if err != nil {
				fmt.Println(err)
			}
			fmt.Println(*user.Username)

			file, err := cmd.Flags().GetString("from-file")
			if len(file) > 0 && err == nil {
				// Open the file
				csvfile, err := os.Open(file)
				if err != nil {
					fmt.Println(err)
				}

				// Parse the file
				r := csv.NewReader(csvfile)
				//r := csv.NewReader(bufio.NewReader(csvfile))

				// Iterate through the records
				for {
					// Read each record from csv
					raw, err := r.Read()
					if err == io.EOF {
						break
					}
					if err != nil {
						log.Fatal(err)
					}
					enabled := true

					user.Email = &raw[3]
					user.Username = &raw[2]
					user.FirstName = &raw[0]
					user.LastName = &raw[1]
					user.Enabled = &enabled

					res, err := rCli.GetClient().CreateUser(ctx, token.AccessToken, rCli.Name, user)

					if err != nil {
						fmt.Println("Oh no!, failed to create user :(")
					}
					pass, err := password.Generate(8, 0, 0, false, true)
					if err != nil {
						fmt.Println(err)
					}
					if err = rCli.GetClient().SetPassword(ctx, token.AccessToken, res, rCli.Name, pass, true); err != nil {
						fmt.Println(err)
					}

				}
				return
			}

			res, err := rCli.GetClient().CreateUser(ctx, token.AccessToken, rCli.Name, user)
			if err != nil {
				fmt.Println(err)
			}
			pass, err := password.Generate(8, 0, 0, true, false)
			if err != nil {
				fmt.Println(err)
			}
			if err = rCli.GetClient().SetPassword(ctx, token.AccessToken, res, rCli.Name, pass, true); err != nil {
				fmt.Println(err)
			}
		},
	}

	cmd.Flags().StringVarP(user.Username, "username", "", "", "")
	cmd.Flags().BoolVarP(user.Enabled, "enabled", "", true, "")
	cmd.Flags().StringVarP(user.FirstName, "first-name", "", "", "")
	cmd.Flags().StringVarP(user.LastName, "last-name", "", "", "")
	cmd.Flags().StringVarP(user.Email, "email", "", "", "")
	//	cmd.Flags().BoolVarP(user.EmailVerified, "email-verified", "", true, "")

	cmd.Flags().StringP("from-file", "", "", "")

	return cmd
}
